#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#include "mirror.h"
#include "config.h"
#include "misc.h"
#include "sockets.h"
#include "compare.h"
#include "log.h"


char server_config[128] = "mirror.cfg";
static FILE *fd = NULL;



enum yesno_type config_yesno(char *s)
{

  if (!mystrccmp(s, "yes"))
    return YN_YES;
  else return YN_NO;

}


enum yesno_type config_yesnoauto(char *s)
{

  if (!mystrccmp(s, "yes"))
    return YN_YES;
  else if (!mystrccmp(s, "auto"))
    return YN_AUTO;
  else return YN_NO;

}





void config_read(mirror_t *Mirror)
{
  char line[256], *ar, *token, *value;
  int linenum = 0;

  printf("Reading '%s'...\n", server_config);


  fd = fopen(server_config, "rb");

  if (!fd) {
    perror("fopen()");
    exit(1);
  }


#ifdef LOCK_EX
  // Lock the file to ensure we only have one process working on this cfg.

  if ( flock( fileno( fd ), LOCK_EX | LOCK_NB ) ) {

    printf("Config file '%s' is locked.\nPossibly another mirror process is already running?\n\n", server_config);

    exit(0);

  }

#endif





  while(fgets( line, 256, fd )) {

    linenum++;

    if (line[0] == '#') continue;


    ar = line;


    token = misc_digtoken( &ar, " \t\r\n");
    if (!token) continue;


    if (!mystrccmp(token, "A_name:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.name = mystrcpy(value);


    } else if (!mystrccmp(token, "A_host:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.host = mystrcpy(value);


    } else if (!mystrccmp(token, "A_port:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.port = atoi(value);


    } else if (!mystrccmp(token, "A_user:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.user = mystrcpy(value);


    } else if (!mystrccmp(token, "A_pass:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.pass = mystrcpy(value);


    } else if (!mystrccmp(token, "A_passive:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.passive = config_yesno(value);



    } else if (!mystrccmp(token, "A_onlypasv:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.onlypasv = config_yesnoauto(value);


    } else if (!mystrccmp(token, "A_initial:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.initial = config_yesno(value);


    } else if (!mystrccmp(token, "A_timeout:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.timeout = atoi(value);


    } else if (!mystrccmp(token, "A_timeout_transfer:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.timeout_transfer = atoi(value);


    } else if (!mystrccmp(token, "A_retries:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.retries = atoi(value);


    } else if (!mystrccmp(token, "A_pause:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.pause = atoi(value);


    } else if (!mystrccmp(token, "A_sleep:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.sleep = atoi(value);


    } else if (!mystrccmp(token, "A_log:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      log_sopen(&Mirror->A_host, value);


    } else if (!mystrccmp(token, "A_verbose_list:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.verbose_dir = config_yesno(value);


    } else if (!mystrccmp(token, "A_save_file:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->A_host.save_file = mystrcpy(value);


    } else if (!mystrccmp(token, "B_name:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.name = mystrcpy(value);


    } else if (!mystrccmp(token, "B_host:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.host = mystrcpy(value);


    } else if (!mystrccmp(token, "B_port:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.port = atoi(value);


    } else if (!mystrccmp(token, "B_user:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.user = mystrcpy(value);


    } else if (!mystrccmp(token, "B_pass:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.pass = mystrcpy(value);


    } else if (!mystrccmp(token, "B_passive:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.passive = config_yesno(value);



    } else if (!mystrccmp(token, "B_onlypasv:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.onlypasv = config_yesnoauto(value);


    } else if (!mystrccmp(token, "B_initial:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.initial = config_yesno(value);


    } else if (!mystrccmp(token, "B_timeout:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.timeout = atoi(value);


    } else if (!mystrccmp(token, "B_timeout_transfer:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.timeout_transfer = atoi(value);


    } else if (!mystrccmp(token, "B_retries:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.retries = atoi(value);


    } else if (!mystrccmp(token, "B_pause:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.pause = atoi(value);


    } else if (!mystrccmp(token, "B_sleep:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.sleep = atoi(value);

    } else if (!mystrccmp(token, "B_log:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      log_sopen(&Mirror->B_host, value);


    } else if (!mystrccmp(token, "B_verbose_list:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.verbose_dir = config_yesno(value);


    } else if (!mystrccmp(token, "B_save_file:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->B_host.save_file = mystrcpy(value);


    } else if (!mystrccmp(token, "mirror:")) {

      if (!config_getmirror( Mirror, &ar, linenum )) goto parse_error; 




    } else if (!mystrccmp(token, "max_dir_age:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->max_dir_age = (time_t) atol(value);

    } else if (!mystrccmp(token, "max_file_age:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->max_file_age = (time_t) atol(value);

    } else if (!mystrccmp(token, "move_empty_files:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->move_empty_files = config_yesno(value);

    } else if (!mystrccmp(token, "move_empty_dirs:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      Mirror->move_empty_dirs = config_yesno(value);


    } else if (!mystrccmp(token, "move_first:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      compare_add2first( value );


    } else if (!mystrccmp(token, "ignore_name:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      compare_add2ignore( value );


    } else if (!mystrccmp(token, "mirror_log:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      log_open( LOG_MIRROR, value );


    } else if (!mystrccmp(token, "debug_log:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      log_open( LOG_DEBUG, value );


    } else if (!mystrccmp(token, "error_log:")) {

      if (!(value = misc_digtoken( &ar, " \t\r\n"))) goto parse_error;
      
      log_open( LOG_ERROR, value );



    } else {

      printf("Unknown keyword '%s' at line %d -- ignored\n", token, linenum);

    }


    continue;  // So it doesn't enter the parse error bit

  parse_error:
    printf("Parse error at line %d -- token '%s' requires argument...\n",
	   linenum, token);


  } // while(fgets)




  //  fclose(fd);



  // Look up hostnames, if any.

  Mirror->A_host.ip = sockets_gethost( Mirror->A_host.host );
  Mirror->B_host.ip = sockets_gethost( Mirror->B_host.host );

  if (!Mirror->A_host.ip || !Mirror->B_host.ip) {
    exit(1);
  }

  printf("A_host %s:%d\n",
	 ul2a(Mirror->A_host.ip), Mirror->A_host.port);
  printf("B_host %s:%d\n",
	 ul2a(Mirror->B_host.ip), Mirror->B_host.port);



}




void config_free(void)
{

  if (fd) {
#ifdef LOCK_UN
    flock( fileno( fd ), LOCK_UN);
#endif
    fclose(fd);

    fd = NULL;

  }


}





//
// read a mirror: argument

int config_getmirror(mirror_t *mirror, char **ar, int line)
{
  char *token, *r;
  path_t newp, *node;


  memset(&newp, 0, sizeof(newp));


  // Read the bi-directional

  if (!(token = misc_digtoken( ar, " \t\r\n"))) return 0;


  newp.bidirectional = config_yesno( token );


  // Read siteX:/path argument

  if (!(token = misc_digtoken( ar, " \t\r\n"))) return 0;

  if (!(r = strchr(token, ':'))) {
    printf("Malformed mirror: command line %d, missing :\n", line);
    return 0;
  }

  *r = 0;
  r++;


  if (!mystrccmp("siteA", token)) {
    newp.A_path = mystrcpy( r );
  } else if (!mystrccmp("siteB", token)) {
    newp.B_path = mystrcpy( r );
  } else {
    printf("Malformed mirror: command line %d, siteA or siteB expected, not %s\n", 
	   line, token);
    return  0;
  }


  // Read siteY:/path argument

  if (!(token = misc_digtoken( ar, " \t\r\n"))) return 0;

  if (!(r = strchr(token, ':'))) {
    printf("Malformed mirror: command line %d, missing :\n", line);
    return 0;
  }

  *r = 0;
  r++;


  if (!mystrccmp("siteA", token)) {
    newp.A_path = mystrcpy( r );
  } else if (!mystrccmp("siteB", token)) {
    newp.B_path = mystrcpy( r );
  } else {
    printf("Malformed mirror: command line %d, siteA or siteB expected, not %s\n",
	   line, token);
    return  0;
  }



  // Done.

  // If either path are NULL then the user specified siteA: siteA: or both B
  if (!newp.A_path || !newp.B_path) {
    printf("Malformed mirror: command line %d, expecting to see both siteA and siteB in line.\n", line);
    return 0;
  }


  // We parsed it correctly.
  node = (path_t *) malloc(sizeof(newp));

  if (!node) {
    perror("malloc");
    return 0;
  }


  // Copy over
  memcpy(node, &newp, sizeof(newp));

  
  node->next = mirror->paths;
  mirror->paths = node;

  return 1;
}



