#include <stdio.h>


#include "mirror.h"
#include "cps.h"
#include "misc.h"
#include "io.h"



static float global_cps  = 0;         // Last "average" cps over all

static float added_cps   = 0;         // Accumulated cps over...
static int   num_samples = 0;         // .. this many samples

static float file_cps    = 0;         // Last file cps



void cps_new(float cps)
{

  num_samples++;
  added_cps += cps;

  file_cps = cps;


  global_cps = added_cps / num_samples;

}




time_t cps_time(mirror_t *mirror, time_t eta)
{
  time_t elapsed;


  elapsed = io_global_time - mirror->A_host.start;

  if (elapsed > eta) return 0;

  return eta - elapsed;

}



void cps_display(mirror_t *mirror)
{


  if (!mirror->A_host.mirror_current ||
      !mirror->A_host.mirror_current->size) return;

  if ((mirror->A_host.state.state != ST_MIRROR) ||
      (mirror->A_host.state.substate != 18)) return;

  if (!num_samples) return;



  //  printf("Avg KB/sec  |Last KB/sec  |File ETA    |Dir ETA     |Est. %%\n");
  printf("  %7.2f   |  %7.2f    |%11.11s |%11.11s |  %-3d%%  \r",
	 global_cps / 1024.0, 
	 file_cps / 1024.0,
	 misc_idletime(
		       cps_time(
				mirror, 
				mirror->A_host.mirror_current->size /
				global_cps)),

	 misc_idletime2(
			cps_time(
				 mirror,
				 cps_dirsize(mirror) / 
				 global_cps)),
	 cps_percent(mirror));

  fflush(stdout);

}




float cps_dirsize(mirror_t *mirror)
{
  float result;
  dir_t *runner;


  result = 0.0;


  for (runner = mirror->A_host.mirror_current;
       runner;
       runner = runner->next) {

    result += (float) runner->size;

  }

  return result;

}


int cps_percent(mirror_t *mirror)
{
  time_t elapsed;
  unsigned long bytes;
  float percent;

  // Estimate the amount of bytes we have transfered

  // Time elapsed
  elapsed = io_global_time - mirror->A_host.start;


  // Estimate bytes transfered
  bytes = global_cps * elapsed;


  if (bytes >= mirror->A_host.mirror_current->size) return 100;

  
  // Work out percentage
  percent = ((float) bytes) /
    ( (float) mirror->A_host.mirror_current->size); 
    

  percent *= 100.0;

  if (percent >= 100.0) return 100;
  if (percent <=   0.0) return 0;

  return (int) percent;

}


float cps_getavg(void)
{

  return (global_cps / 1024.0);

}


