#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <limits.h>

#include "mirror.h"
#include "date.h"
#include "log.h"
#include "cps.h"
#include "misc.h"


void date_stamp(mirror_t *mirror, site_t *node, dir_t *file)
{
  FILE *fd;
  path_t *runner;


  if (!node->save_file) {
    log_printf(LOG_ERROR, "No %s save_file set - can't record date.\n", 
	       node->name);
    return;
  }

  fd = fopen( node->save_file, "wb");

  if (!fd) {
    perror("fopen");
    log_printf(LOG_ERROR, "Failed to open %s's save_file.\n",
	       node->name);
    return;
  }


  fprintf(fd, "Last_Date: %lu\r\n" , (unsigned long)file->date);
  fprintf(fd, "Last_Dir:  \"%s\"\r\n" , file->name);
  fprintf(fd, "Last_Cps:  %7.2f\r\n" , cps_getavg());


  for (runner = mirror->paths; runner; runner = runner->next) {
    
    if (!mystrccmp(runner->A_path, node->current_path->A_path)) {
      
      if (file->date >= runner->stamp) {
	
	runner->stamp = file->date;
	mirror->site_max_age = file->date;
	log_printf(LOG_DEBUG, "Finished '%s' - recording date %s",
		   node->mirror_current ? node->mirror_current->name : file->name,
		   ctime(&file->date));
	
      }

    }
    
    fprintf(fd, "Last_Mirror: \"%s\" %lu\r\n", runner->A_path,
	    (unsigned long)runner->stamp);
    
  }
  
  fclose(fd);
  
  
  
}




void date_load(mirror_t *mirror, site_t *node)
{
  FILE *fd;
  char buffer[1024], *token, *ar;
  time_t last_date = 0;
  path_t *runner;

  // Attempt to load the date stored.
  // If this date is newer than max_age, set max_age to it.
  // if it is older, keep max_age.

  if (node->current_path && node->current_path->A_path) 
    log_printf(LOG_DEBUG, "Looking for '%s' date stamp\n", node->current_path->A_path);


  fd = NULL;

  if (node->save_file)
    fd = fopen(node->save_file, "rb");

  if (!fd) {

    // No save file, check initial.
    log_printf(LOG_DEBUG, "No %s save_file.\n", node->name);

    date_initial(mirror, node);

    return;
  }


  while(fgets(buffer, 1024, fd)) {

    ar = buffer;

    if (!(token = misc_digtoken(&ar, " \t\r\n")))
      continue;

    if (!mystrccmp("Last_Date:", token)) {

      if (!(token = misc_digtoken(&ar, " \t\r\n")))
	continue;

      last_date = strtoul(token, NULL, 10);


    } else if (!mystrccmp("Last_Mirror:", token)) {
      
      if (!(token = misc_digtoken(&ar, "\"")))
	continue;
      
      for (runner = mirror->paths; runner; runner = runner->next) {
	
	if (!mystrccmp(runner->A_path, token)) {
	  
	  if (!(token = misc_digtoken(&ar, " \t\r\n")))
	    continue;
	  
	  last_date = strtoul(token, NULL, 10);
	  runner->stamp = last_date;
	  
	  log_printf(LOG_DEBUG, "Date Stamp for '%s' read as %lu\n", 
		 runner->A_path,
		 last_date);
	  
	}
	
	//      last_date = strtoul(token, NULL, 10);
      }

    }

  }

  fclose(fd);

  if (!last_date) {
    date_initial(mirror, node);
    return;
  }


  mirror->site_max_age = last_date;

  log_printf( LOG_DEBUG, "Setting site_max_age to %lu\n", last_date);

}





void date_initial(mirror_t *mirror, site_t *node)
{

  // Check if initial is set.

  if (node->initial == YN_NO) {

    if (!node->skip_initial)
      log_printf(LOG_DEBUG, "Initial run only. Retreiving date information\n");

    node->skip_initial = 1;


  }

}
