#include <stdio.h>
#include <string.h>

#include "site.h"
#include "fxp.h"


//
// Send PASV/PORT command depending on the switches
//
void fxp_init    (mirror_t *mirror)
{

  // send PASV to siteA and PORT to B
  // there is only one case we swap it, and that is if
  // siteA has onlypasv no, or site B has onlypasv yes.

  if ((mirror->A_host.onlypasv == YN_NO) ||
      (mirror->B_host.onlypasv == YN_YES)) { // reverse it

    site_printf(&mirror->B_host, "PASV\r\n");

  } else {

    site_printf(&mirror->A_host, "PASV\r\n");

  }

}





int  fxp_initdone(mirror_t *mirror)
{
  
  // Wait for passive reply.
  if (mirror->A_host.cmd_reply == 227) {
    fxp_sendport(&mirror->B_host, mirror->A_host.cmd);
    mirror->A_host.cmd_reply = 0;
  }

  if (mirror->B_host.cmd_reply == 227) {
    fxp_sendport(&mirror->A_host, mirror->B_host.cmd);
    mirror->B_host.cmd_reply = 0;
  }

  if (mirror->A_host.cmd_reply == 425) 
    return -1;

  if (mirror->B_host.cmd_reply == 425) 
    return -1;

  // check for 200 PORT
  if ((mirror->A_host.cmd_reply == 200) &&
      !strncmp("200 PORT", mirror->A_host.cmd, 8)) {
    return 1;
  }

  if ((mirror->B_host.cmd_reply == 200) &&
      !strncmp("200 PORT", mirror->B_host.cmd, 8)) {
    return 1;
  }

  return 0;

}






void fxp_xfer(mirror_t *mirror)
{

  // send STOR to siteB
  // send RETR to siteA


  if (mirror->B_host.restart)
    site_printf(&mirror->B_host, "APPE %s\r\n", 
		mirror->A_host.mirror_current->name);
  else
    site_printf(&mirror->B_host, "STOR %s\r\n", 
		mirror->A_host.mirror_current->name);


  site_printf(&mirror->A_host, "RETR %s\r\n", 
	      mirror->A_host.mirror_current->name);


}







void fxp_sendport(site_t *node, char *line)
{

  char *p1, *p2;
  
  // line should have "227 Entering Passive Mode (127,0,0,1,255,21)"


  if (!(p1 = strchr(line, '('))) {
    site_printf(node, "PORT dummy\r\n");
    return;
  }

  if (!(p2 = strchr(line, ')'))) {
    site_printf(node, "PORT dummy\r\n");
    return;
  }

  p1++;
  *p2 = 0;

  site_printf(node, "PORT %s\r\n", p1);

}



void fxp_port    (site_t *node)
{
  unsigned char *pp;
  int IPA[4], PORTA[2];
  
  pp = (unsigned char *)&(node->pasv_host);
  IPA[0] = (unsigned char) pp[0];
  IPA[1] = (unsigned char) pp[1];
  IPA[2] = (unsigned char) pp[2];
  IPA[3] = (unsigned char) pp[3];
  
  
  pp = (unsigned char *)&(node->pasv_port);
  PORTA[0] = pp[0];
  PORTA[1] = pp[1];
  
  site_printf(node, "PORT %d,%d,%d,%d,%d,%d\r\n", 
	     IPA[0], IPA[1], IPA[2], IPA[3], PORTA[0], PORTA[1]);
  

}
