#include <stdio.h>
#if __STDC__ || WIN32
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#ifndef WIN32
#include <unistd.h>
#endif

#include "mirror.h"
#include "log.h"
#include "misc.h"


static FILE *debug_fd  = NULL;
static FILE *error_fd  = NULL;
static FILE *mirror_fd = NULL;






void log_sopen(site_t *node, char *filename)
{
  FILE *fd;

  fd = NULL;

  if (node->log) {
    log_sclose(node);
  }


  if (!mystrccmp("stdout", filename)) {

    fd = stdout;

  } else {

    fd = fopen(filename, "ab");

  }

  
  if (!fd) {
    perror("log_sopen");
    return;
  }


  setvbuf(fd, NULL, _IOLBF, 0);


  node->log = fd;

}



void  log_sclose   (site_t *node)
{

  if (node->log) {

    fclose(node->log);
    node->log = NULL;

  }

}




#if __STDC__ || WIN32
void log_sprintf(site_t *node, char const *fmt, ...)
#else
void log_sprintf(node, fmt, va_alist)
     site_t *node;
     char const *fmt;
     va_dcl
#endif
{
  va_list ap;
  char msg[1024];


  if (!node->log) return;


#if __STDC__ || WIN32
  va_start(ap, fmt);
#else
  va_start(ap);
#endif

  // Buffer overflow! No vsnprintf on many systems :(
  vsprintf(msg, fmt, ap);
  va_end(ap);

  
  fputs(msg, node->log);
                 
}




FILE *log_type2file(enum log_type x)
{

  switch (x) {
  case LOG_DEBUG:
    return debug_fd;
  case LOG_ERROR:
    return error_fd;
  case LOG_MIRROR:
    return mirror_fd;
  default:
    break;
  }
  return NULL;
}


void log_settype2file(enum log_type x, FILE *set)
{

  switch (x) {
  case LOG_DEBUG:
    debug_fd = set;
    break;
  case LOG_ERROR:
    error_fd = set;
    break;
  case LOG_MIRROR:
    mirror_fd = set;
    break;
  default:
    break;
  }
}










void log_open(enum log_type type, char *filename)
{
  FILE *fd;

  fd = log_type2file(type);

  if (fd) {
    log_close(type);
  }


  if (!mystrccmp("stdout", filename)) {

    fd = stdout;

  } else {

    fd = fopen(filename, "ab");
    

  }

  
  if (!fd) {
    perror("log_open");
    return;
  }

  setvbuf(fd, NULL, _IOLBF, 0);

  log_settype2file(type, fd);


  // If we set LOG_ERROR, and it's not to stdout, then
  // also set stderr to the same fd.
#ifndef WIN32
  if ((type == LOG_ERROR) && (fd != stdout)) {

    dup2(fileno(stderr), fileno(fd));

  }
#endif

}



void log_close(enum log_type type)
{
  FILE *fd;

  fd = log_type2file(type);

  if (fd) {

    fclose(fd);
    log_settype2file(type, NULL);

  }

}




#if __STDC__ || WIN32
void log_printf(enum log_type type, char const *fmt, ...)
#else
void log_printf(type, fmt, va_alist)
     enum log_type type;
     char const *fmt;
     va_dcl
#endif
{
  va_list ap;
  char msg[1024];
  FILE *fd;


  fd = log_type2file(type);

  if (!fd) return;


#if __STDC__ || WIN32
  va_start(ap, fmt);
#else
  va_start(ap);
#endif

  // Buffer overflow! No vsnprintf on many systems :(
  vsprintf(msg, fmt, ap);
  va_end(ap);

  
  fputs(msg, fd);
                 
}









