#include <stdio.h>
#include <stdlib.h>
#include <signal.h>


#include "mirror.h"
#include "config.h"
#include "arguments.h"
#include "io.h"
#include "state.h"
#include "sockets.h"
#include "date.h"
#include "misc.h"
#include "fnmatch.h"

#include "version.h"
#include "stats.h"


static mirror_t Mirror;




int mirror_verbose = 0;




int main(int argc, char **argv)
{

  printf("Version 1.13 build %lu (%s@%s) - %s\n", 
	 (unsigned long) VERSION_BUILD, VERSION_USER, VERSION_HOST, 
	 VERSION_DATE);
  printf("lmirror by Jorgen Lundman <lundman@lundman.net>\n\n");


  memset(&Mirror, 0, sizeof(Mirror));

  stats_init();

  parse_arguments( argc, argv);

  
  config_read(&Mirror);

  
  time(&io_global_time);

  //  stats_display(&Mirror.A_host);

  state_reset(&Mirror.A_host);
  state_reset(&Mirror.B_host);


  if (sockets_init()) {
    printf("Failed to initialise sockets\n");
    exit(0);
  }


  date_load(&Mirror, &Mirror.A_host);
  //  date_load(&Mirror, &Mirror.B_host);

#ifndef WIN32
  signal(SIGPIPE, SIG_IGN);
#endif



  io_loop(&Mirror);


  sockets_exit();


  config_free();


  printf("[--== Statistics ==--]\n");
  printf("        Site A\n");
  stats_display(&Mirror.A_host);
  printf("        Site B\n");
  stats_display(&Mirror.B_host);

  exit(0);

}






void mirror_addmanual(char *dir)
{
  dir_t *newd;


  newd = (dir_t *) malloc( sizeof (dir_t));

  if (!newd) return;
  

  memset(newd, 0, sizeof(dir_t));

  
  newd->name = mystrcpy( dir );

  newd->next = Mirror.manual_dirs;
  Mirror.manual_dirs = newd;



}



int mirror_ismanual(dir_t *dir)
{
  dir_t *runner;

  for (runner = Mirror.manual_dirs; runner; runner = runner->next) {

    if (!fnmatch(runner->name, dir->name, FNM_CASEFOLD)) {

      return 1;

    }
  }
  return 0;
}
