#ifndef MIRROR_H
#define MIRROR_H

#include <time.h>


/* Defines */

#define DATA_BUFSZ 8192
#define MAX_CMD_QUEUE 5


/* Variables */

enum yesno_type { YN_NO,   YN_YES,       YN_AUTO };
enum dir_type   { DT_FILE, DT_DIRECTORY, DT_LINK,    DT_SPECIAL, DT_OTHER };
enum state_type { ST_NONE, ST_LOGIN,     ST_COLLECT, ST_COMPARE, ST_MIRROR, ST_DISCONNECT, ST_RETRY, ST_DONE };
enum socket_type { SO_NONE, SO_PENDING, SO_LISTEN, SO_ACTIVE, SO_BUFFERFULL, SO_ERROR };


extern int mirror_verbose;



struct dir_struct {

  struct dir_struct *parent;   // Null if top

  struct dir_struct *next;     // Next entry in "same level" (current dir)

  struct dir_struct *subdir;   // Subdirectory if any (if we are a dir)

  enum dir_type      type;     // file, directory

  char              *name;
  unsigned long      size;     // wont work with files > 4G

  time_t             date;

  char              *dest_name;

};

typedef struct dir_struct dir_t;



struct path_struct {
  char  *A_path;
  char  *B_path;

  enum yesno_type bidirectional;

  dir_t *A_queue;
  dir_t *B_queue; // Only filled in for bi-directional mirroring.


  time_t stamp;   // date stamp


  struct path_struct *next;
};

typedef struct path_struct path_t;


struct state_struct {
  
  enum state_type   state;
  int               substate;
  enum socket_type  socket;
  enum socket_type  pasv;

};

typedef struct state_struct state_t;

struct stats_s {
  unsigned MY_LONG64 bytes_site_to_site;

  unsigned long bytes_from;
  unsigned long bytes_to;

  unsigned int  total_directories;
  unsigned int  total_files;

  unsigned int  skipped_files;

  float    avg_cps;

  time_t   start_time;

} ;


struct site_struct {
  
  char            *name;
  char            *host;         // Host string
  unsigned long    ip;           // Network byte-order IP
  unsigned int     port;         // Note it is not short.
  char            *user;
  char            *pass;
  enum yesno_type  passive;
  enum yesno_type  onlypasv;
  enum yesno_type  initial;
  unsigned int     timeout;
  unsigned int     timeout_transfer;
  unsigned int     retries;
  unsigned int     pause;
  unsigned int     sleep;
  enum yesno_type  verbose_dir;

  unsigned int     retries_done;

  int              socket;

  unsigned int     cmd_reply;
  unsigned char   *cmd;
  unsigned int     cmd_queue[MAX_CMD_QUEUE];
  unsigned int     cmd_offset;

  state_t          state;

  path_t          *current_path;
  char            *current_cwd;
  dir_t           *current_dir;

  dir_t           *mirror_paths;
  dir_t           *mirror_current;

  unsigned long    restart;

  time_t           idle;
  time_t           start;

  unsigned long    pasv_host;
  unsigned short int pasv_port;
  int              pasv_socket;


  FILE            *log;
  char            *save_file;

  int              skip_initial;


  unsigned char    inbuffer[ DATA_BUFSZ ];
  int              in_buffer;

  unsigned char    outbuffer[ DATA_BUFSZ ];
  int              out_buffer;

  unsigned char    pasvbuffer[ DATA_BUFSZ ];
  int              pasv_buffer;


  struct stats_s   stats;


};

typedef struct site_struct site_t;



struct mirror_struct {

  site_t  A_host;
  site_t  B_host;

  // Mirror paths here.
  path_t *paths;

  time_t  site_max_age;
  time_t  max_dir_age;
  time_t  max_file_age;

  enum yesno_type move_empty_files;
  enum yesno_type move_empty_dirs;


  dir_t *manual_dirs;   // List of manual entries, if any

};


typedef struct mirror_struct mirror_t;




/* Functions */

int  mirror_ismanual (dir_t *);
void mirror_addmanual(char *);



#endif

