// $Id: misc.c,v 1.2 2000/03/16 14:03:54 lundman Exp $
// Miscellaneous utility functions
// Jorgen Lundman November 25th, 1999
// Copyright (c) 1999 HotGen Studios Ltd <www.hotgen.com>

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>


#include "misc.h"

// Filled in with the matched character
char misc_digtoken_optchar = 0;


char *misc_digtoken(char **string,char *match) 
{
  if(string && *string && **string) {
    
    while(**string && strchr(match,**string))
      (*string)++;
    
    if(**string) { /* got something */
      char *token=*string;
      
      if((*string=strpbrk(*string,match))) {
	
	misc_digtoken_optchar = *(*string);

        *(*string)++=(char)0;
	while(**string && strchr(match,**string))
          (*string)++;
	
      }  else
        *string = ""; /* must be at the end */
      
      return(token);
    }
  }
  return((char *)0);
}



//
// If you want to check out unreadable code, check this out
//
int mystrccmp(register char *s1,register char *s2) {

  while((((*s1)>='a'&&(*s1)<='z')?(*s1)-32:*s1)==(((*s2)>='a'&&(*s2)<='z')?(*s2++)-32:*s2++))
    if(*s1++==0) return 0;
  return (*(unsigned char *)s1-*(unsigned char *)--s2);
}



char *mystrcpy(char *s)
{
  char *r;

  r = (char *) malloc(strlen(s)+1);
  if (!r) {
    perror("mystrcpy()");
    exit(1);
  }

  strcpy(r, s);
  return r;
}



// switch uses jump tables so they are O(0)
#define CONVCHAR(X, Y) switch((X)) {\
                    case '1':    \
                    case '2':    \
                    case '3':    \
                    case '4':    \
                    case '5':    \
                    case '6':    \
                    case '7':    \
                    case '8':    \
                    case '9':    \
                    case '0':    \
                      (Y)=(X)-'0';   \
                      break;     \
                    case 'a':    \
                    case 'b':    \
                    case 'c':    \
                    case 'd':    \
                    case 'e':    \
                    case 'f':    \
                      (Y)=(X)-'a'+10;\
                      break;     \
                    case 'A':    \
                    case 'B':    \
                    case 'C':    \
                    case 'D':    \
                    case 'E':    \
                    case 'F':    \
                      (Y)=(X)-'A'+10;\
                      break;     \
                    }


//
// Convert a hex string "4f32109fae" into the equivalent binary, return
// number of bytes
//
int  misc_hextobin(char *hex)
{
  unsigned char high, low;
  int result, out;

  result = 0;
  out = 0;

  while (hex[out]) {

    CONVCHAR(hex[out], high);
    out++;
    CONVCHAR(hex[out], low);
    out++;
    
    hex[result] = (high << 4) + low;
    result ++;
    
  }

  return result;

}


//
// Convert binary into hex string, reverse order so we can re-use memory
//
int  misc_bintohex(char *bin, int len)
{
  unsigned char hex, low, high;
  unsigned char table[16] = 
    {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};

  bin[len * 2] = 0; // terminate string


  for (len--; len >= 0; len--) {

    hex = bin[ len ];

    low = hex & 0x0f;
    high = (hex & 0xf0) >> 4;

    bin[ len * 2 + 1] = table[ low ];
    bin[ len * 2 ]    = table[ high  ];

  }

  return 0;

}



char *misc_strdup(char *s)
{
  char *result;

  result = (char *) malloc( strlen( s ) +1 );

  if (result)
    strcpy(result, s);

  return result;

}



time_t misc_getdate(char *date)
{
  time_t now;
  struct tm tmw, *tmn;
  char *ar, *token, *r;
  int i;
  static char months[12][4] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

  // It is either
  // 'Nov 10 11:44'   for less than 6mnths old or
  // 'Mar 27  1999'   for more.

  memset(&tmw, 0, sizeof(tmw));
  ar = date;


  // read month
  if (!(token = misc_digtoken(&ar, " "))) goto getdate_error;

  for (i = 0; i < 12; i++)
    if (!mystrccmp(months[i], token)) {
      tmw.tm_mon = i;
      break;
    }

  if (i == 12) goto getdate_error;


  // get day of month
  if (!(token = misc_digtoken(&ar, " "))) goto getdate_error;

  tmw.tm_mday = atoi(token);


  // get either time, or, year
  if (!(token = misc_digtoken(&ar, " "))) goto getdate_error;


  if ((r = strchr(token, ':'))) {  
    
    // it's time
    *r = 0;


    time(&now);
    tmn = localtime(&now);
    if (!tmn) goto getdate_error;
    
    tmw.tm_hour = atoi(token);
    tmw.tm_min  = atoi(&r[1]);
    tmw.tm_year = tmn->tm_year;


  } else { 

    // It's the year
    tmw.tm_year = atoi(token) - 1900; // as according to the man page

  }


  // Convert it into a time_t, well, as close as possible.
  return mktime(&tmw);

 getdate_error:
  return 0;

}


char *misc_idletime(time_t time)
{ /* This support functionw as written by George Shearer (Dr_Delete) */
  
  static char workstr[100];
  unsigned short int days=(time/86400),hours,mins,secs;
  hours=((time-(days*86400))/3600);
  mins=((time-(days*86400)-(hours*3600))/60);
  secs=(time-(days*86400)-(hours*3600)-(mins*60));
  
  workstr[0]=(char)0;
  if(days)
    sprintf(workstr,"%dd",days);
  sprintf(workstr,"%s%s%02d",workstr,(workstr[0])?":":"",hours);
  sprintf(workstr,"%s%s%02d",workstr,(workstr[0])?":":"",mins);
  sprintf(workstr,"%s%s%02d",workstr,(workstr[0])?":":"",secs);
  if (!days && !hours && !mins && !secs)
    sprintf(workstr,"0 seconds");
  
  return(workstr);
}

char *misc_idletime2(time_t time)
{ /* This support functionw as written by George Shearer (Dr_Delete) */
  
  static char workstr[100];
  unsigned short int days=(time/86400),hours,mins,secs;
  hours=((time-(days*86400))/3600);
  mins=((time-(days*86400)-(hours*3600))/60);
  secs=(time-(days*86400)-(hours*3600)-(mins*60));
  
  workstr[0]=(char)0;
  if(days)
    sprintf(workstr,"%dd",days);
  sprintf(workstr,"%s%s%02d",workstr,(workstr[0])?":":"",hours);
  sprintf(workstr,"%s%s%02d",workstr,(workstr[0])?":":"",mins);
  sprintf(workstr,"%s%s%02d",workstr,(workstr[0])?":":"",secs);
  if (!days && !hours && !mins && !secs)
    sprintf(workstr,"0 seconds");
  
  return(workstr);
}
