#include <stdio.h>
#include <stdlib.h>       // malloc
#include <string.h>       // memset
#if __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif
#include <ctype.h>
#include <errno.h>


#include "pasv.h"
#include "mirror.h"
#include "sockets.h"
#include "state.h"
#include "misc.h"
#include "name.h"
#include "io.h"
#include "compare.h"
#include "log.h"


int pasv_input(site_t *node)
{
  int result;
  char *line;   // Pointer to the current start of a string.


  // While we can get a line from the buffer parse it.
  // notice this can get seperated from here, so that we
  // simply call read() and fill the buffer here, and call
  // io_getline from elsewere. But not really required.

  // Reset the current position counter here.
  line = NULL;

  while((result = pasv_getline(node, &line)) > 0) { 

    // Pass the string on to next layer
    pasv_command(node, line);

  }

  // We are done.
  // We return 0 when there are no more info to process
  // and -1 signifies the connection was lost.

  return result;


}





int pasv_getline(site_t *node, char **last)
{ 
  int len;                      // return int of read().
  int left;                     // remaining items
  static char *next;            // strtok style PC holder. 



  // If last is NULL, it's the first call to this function for this
  // current iteration, so first we call read() to receive more data.
  if (!*last) {


    // Set it to start of the buffer
    *last = node->pasvbuffer;
    next = *last;


    // Retrieve more data from the socket.
    // We write it after any data that may already be left in the buffer
    // from a previous read. Generally doesn't happen, but we need to
    // allocate for it. (Buffer over/under -run hacking etc)
    len = sockets_read(node->pasv_socket, 
		       &node->pasvbuffer[node->pasv_buffer], 
		       DATA_BUFSZ - node->pasv_buffer);

#ifdef DEBUGX
    printf("read %d  >>'", len);
    write(fileno(stdout), &node->pasvbuffer[node->pasv_buffer], len);
    printf("'<<\n");
    fflush(stdout);
#endif

    if (len <= 0) {

      
      node->idle = io_global_time;


      if (!len) {

	log_printf(LOG_DEBUG, "Data connection to %s closed\n", node->name);

	// graceful exit - we are done.
	node->state.pasv = SO_NONE;
	return 0;

      }

#if 0
      if (errno == EAGAIN) {
	printf("EAGAIN returned? shouldn't happen\n");
	return 0;
      }
#endif


      perror("read");

      log_printf(LOG_ERROR, "Data connection to %s lost (%d)\n", node->name, len);


      node->state.pasv = SO_ERROR;

      return -1;
    }
    // Move along the PC according to how much was read.
    node->pasv_buffer += len;

    node->stats.bytes_from += len;


  } else {  // last == NULL

    // last isn't NULL, so we are now still parsing buffer content, if any

    *last = next;

  }


  // Right we now have some data in the buffer, extract a string.

  do {


    if ((int)(next - (char *)node->pasvbuffer) >= node->pasv_buffer) {
      

      // Compute what is remaining
      left = (node->pasv_buffer - (*last - (char *)node->pasvbuffer));


      // we only bother keeping the remaining if it is more than
      // 3, since 2 is just \r\n.

      if (left > 0) {

#ifdef DEBUGX
      printf("Copying remainder of %d bytes, starts with %d %c\n",
	     left, *last[0], *last[0]);
#endif
        
        // Copy remaining.
        memcpy(node->pasvbuffer, *last, left);

        // Reset pointers and PC
        node->pasv_buffer = left;
        next = NULL;
      } else {

        // Ignore the remainder. (It's just \r\n or a combination of those)
        //        node->inbuffer = 0;

      }

      // stop processing
      return 0;
    }


    // Check if we've hit the end.
    if ((*next == '\n') || (*next == '\r')) { /* Got a line */

      // yep, skip past all blank \r or \n.
      while((*next == '\n') || (*next == '\r')) 
        *(next++) = (char) 0;

      // and replace them with \0. (Terminate string).

      // If we've completely finished parsing the input
      // start again from the start. If not, "next" is ready for 
      // next time around.
#ifdef DEBUGX
      printf("inbuf %d last char %d %c\n", node->pasv_buffer,
	     node->pasvbuffer[node->pasv_buffer - 1],
	     node->pasvbuffer[node->pasv_buffer - 1]);
      fflush(stdout);


#endif

      if (node->pasv_buffer - ((int)(next - (char *)node->pasvbuffer)) <= 0) {
        node->pasv_buffer = 0;
      }

      // Signal we have a valid string.
      return 1;
    }

    /* This next should never happen */
    /* But we check for it just incase. I've never seen it yet. */

    if ((next > (char *)node->pasvbuffer + DATA_BUFSZ)) {
      log_printf(LOG_ERROR, "Internal Error, most likely due to a bug\r\n");
      node->pasv_buffer = 0;
      return 0;
    }

    // Advance to next char
    next++;

  } while(1);

  // NOT REACHED
  return 0;  // avoid the warning
}




void pasv_command(site_t *node, char *line)
{
  char *ar, *perm, *thingy, *user, *group, *size, *name;
  char date[14];
  enum dir_type type;
  dir_t *newd;

#if 1
  if (node->verbose_dir == YN_YES)
    log_sprintf(node, "<[%s DATA]: '%s'\n", node->name, line);
#endif

  // drwxr-xr-x   7  root     root         1024 Mar  3 18:36 ..
  // -rw-r--r--   1  ftp      ftp             5 Nov 10 11:42 Return
  // drwxr-xr-x F0   lftpd    lftpd           0 Mar 27  1999 Cd2
  // -rw-r--r-- D0   lftpd    lftpd           0 Mar 27  1999 sample.j
  // drwxrwxrwx   1  user     group           0 Jun 22  1999 old
  // -rwxrwxrwx   1  user     group     5327872 Nov 30  1998 q2-3.20-x86.exe
  // dr-xr-xr-x   1  owner    group           0 Feb 25  0:31 developr
  // -r-xr-xr-x   1  owner    group        7983 Jan 28  1999 dirmap.htm

  ar = line;


  if (!(perm = misc_digtoken(&ar, " \t\r\n"))) goto pasv_error;

  if (!mystrccmp("total", perm)) {  // it's the "total XX" line, ignore it.
    return;
  }

  if (strlen(perm) != 10) { // We assume it's always this length
    log_printf(LOG_ERROR, "[%s] Unknown data, permission section isn't strlen 10 :'%s' - '%s'\n",
		node->name, perm, ar);
    return;
  }

  // Parse out the rest

  if (!(thingy = misc_digtoken(&ar, " \t\r\n"))) goto pasv_error;
  if (!(user   = misc_digtoken(&ar, " \t\r\n"))) goto pasv_error;
  if (!(group  = misc_digtoken(&ar, " \t\r\n"))) goto pasv_error;
  if (!(size   = misc_digtoken(&ar, " \t\r\n"))) goto pasv_error;

  while (*ar == ' ') ar++;  // skip any whitespace to the date section.

  strncpy(date, ar, 12);
  date[12] = 0;

  if (strlen(date) != 12) {
    log_printf(LOG_ERROR, "[%s] Unknown data, date section isn't strlen 12 :'%s' -  %s\n",
	   node->name, date, ar);
    return;
  }


  // The rest is file/dir name.

  name = &ar[12];

  while (*name == ' ') name++;


  if (name_skip(name)) return;



#if 0
  printf("Parsed line: \n");
  printf("\tperm:    '%s'\n", perm);
  printf("\tthingy:  '%s'\n", thingy);
  printf("\tuser:    '%s'\n", user);
  printf("\tgroup:   '%s'\n", group);
  printf("\tsize:    '%s'\n", size);
  printf("\tdate:    '%s'\n", date);
  printf("\tname:    '%s'\n", name);
#endif


  // Check it is either a file or a dir. Ignore all other
  if (tolower(perm[0]) == 'd')
    type = DT_DIRECTORY;
  else if (tolower(perm[0]) == '-')
    type = DT_FILE;
  else {
    // Skip all others
    return;
  }



  newd = (dir_t *) malloc(sizeof(dir_t));

  if (!newd) {
    perror("malloc");
    return;
  }





  memset(newd, 0, sizeof(dir_t));

  
  newd->type = type;
  newd->name = mystrcpy(name);
  newd->size = strtol(size, NULL, 10);
  newd->date = misc_getdate(date);
  


  pasv_addsorted(&(node->current_dir), newd);


  return;

 pasv_error:
  log_printf(LOG_ERROR, "[%s] Failed to parse input '%s'\n", node->name, line);


}



void pasv_addsorted(dir_t **host, dir_t *node)
{
  dir_t *runner, *prev;


  // If the file ends with *.sfv, always add first.
  if (compare_first(node)) {

    node->next = *host;
    *host = node;
    return;

  }



  for( runner = *host, prev = NULL;
       runner; 
       prev = runner, runner = runner->next) {

    // Skip all first nodes
    if (compare_first(runner)) continue;


    //    if (strcmp( runner->name, node->name ) > 0) { // insert here
    if (runner->date > node->date) { // insert here


      if (!prev) {
        node->next = *host;
        *host = node;
      } else {
        node->next = prev->next;
        prev->next = node;
      }

      return;

    }

  }

  // Insert it last
  if (!prev) { // first node
    node->next = *host;
    *host = node;
  } else {
    node->next = prev->next;
    prev->next = node;
  }
}



void pasv_removefirst(dir_t **host)
{
  dir_t *next;


  next = (*host)->next;

  free((*host)->name);

  free(*host);

  *host = next;


}


void  pasv_freedir  (dir_t **node)
{
  dir_t *next;

  while (*node) {

    next = (*node)->next;

    if ((*node)->name) {
      free((*node)->name);
      (*node)->name = NULL;
    }

    if ((*node)->dest_name) {
      free((*node)->dest_name);
      (*node)->dest_name = NULL;
    }
    

    if ((*node)->subdir)
      pasv_freedir(&((*node)->subdir));

    free(*node);

    *node = NULL;

    *node = next;
  }

  *node = NULL;

}



